<?php

/* --------------------------------------------------------------
   ContentSitemapPriority.inc.php 2019-04-12
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2019 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

/**
 * Class ContentSitemapPriority
 *
 * This class represents the localized site map priority
 *
 * @category   System
 * @package    Content
 */
class ContentSitemapPriority implements LocalizedContentAttributeInterface
{
    /**
     * Minimum value
     */
    const MIN = 0.0;
    
    /**
     * Maximum value
     */
    const MAX = 1.0;
    
    
    /**
     * Content site map priority
     *
     * @var \DecimalType
     */
    protected $priority;
    
    /**
     * Language code
     *
     * @var string
     */
    protected $languageCode;
    
    
    /**
     * ContentSitemapPriority constructor
     *
     * @param \DecimalType $priority Content site map priority
     * @param LanguageCode $languageCode Language code
     */
    public function __construct(DecimalType $priority, LanguageCode $languageCode)
    {
        if ($priority->asDecimal() < self::MIN || $priority->asDecimal() > self::MAX) {
            throw new OutOfBoundsException('Invalid priority range');
        }
        
        $this->priority = $this->formatSitemapPriority($priority);
        $this->languageCode = $languageCode->asString();
    }
    
    
    /**
     * Return the content site map priority
     *
     * @return string
     */
    public function content()
    {
        return $this->priority;
    }
    
    
    /**
     * Return the language code
     *
     * @return string
     */
    public function languageCode()
    {
        return $this->languageCode;
    }
    
    
    /**
     * Format the provided site map priority float to its according string representation
     * This method will NOT round the value of a provided float, that contains many decimals
     *
     * @param float $value Float site map priority value
     *
     * @return string
     *
     * @example $number = formatSitemapPriority(0.987654321) // $number === "0.9"
     *
     */
    protected function formatSitemapPriority(DecimalType $value)
    {
        $formatted = floor(($value->asDecimal() * 10)) / 10;
        
        return (string)$formatted;
    }
}